from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workflow_tag_group_response import WorkflowTagGroupResponse


T = TypeVar("T", bound="WorkflowTagResponse")


@_attrs_define
class WorkflowTagResponse:
    """Workflow tag response schema

    Attributes:
        name (str):
        id (UUID):
        organization_id (str):
        order (int):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        description (None | str | Unset):
        emoji (None | str | Unset):
        color (None | str | Unset):
        group_id (None | Unset | UUID): Optional group for mutual exclusivity
        is_archived (bool | Unset):  Default: False.
        workflow_count (int | None | Unset): Number of workflows with this tag (returned in list endpoint)
        group (None | Unset | WorkflowTagGroupResponse): The group this tag belongs to
    """

    name: str
    id: UUID
    organization_id: str
    order: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    description: None | str | Unset = UNSET
    emoji: None | str | Unset = UNSET
    color: None | str | Unset = UNSET
    group_id: None | Unset | UUID = UNSET
    is_archived: bool | Unset = False
    workflow_count: int | None | Unset = UNSET
    group: None | Unset | WorkflowTagGroupResponse = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.workflow_tag_group_response import WorkflowTagGroupResponse

        name = self.name

        id = str(self.id)

        organization_id = self.organization_id

        order = self.order

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        emoji: None | str | Unset
        if isinstance(self.emoji, Unset):
            emoji = UNSET
        else:
            emoji = self.emoji

        color: None | str | Unset
        if isinstance(self.color, Unset):
            color = UNSET
        else:
            color = self.color

        group_id: None | str | Unset
        if isinstance(self.group_id, Unset):
            group_id = UNSET
        elif isinstance(self.group_id, UUID):
            group_id = str(self.group_id)
        else:
            group_id = self.group_id

        is_archived = self.is_archived

        workflow_count: int | None | Unset
        if isinstance(self.workflow_count, Unset):
            workflow_count = UNSET
        else:
            workflow_count = self.workflow_count

        group: dict[str, Any] | None | Unset
        if isinstance(self.group, Unset):
            group = UNSET
        elif isinstance(self.group, WorkflowTagGroupResponse):
            group = self.group.to_dict()
        else:
            group = self.group

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "id": id,
                "organization_id": organization_id,
                "order": order,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if emoji is not UNSET:
            field_dict["emoji"] = emoji
        if color is not UNSET:
            field_dict["color"] = color
        if group_id is not UNSET:
            field_dict["group_id"] = group_id
        if is_archived is not UNSET:
            field_dict["is_archived"] = is_archived
        if workflow_count is not UNSET:
            field_dict["workflow_count"] = workflow_count
        if group is not UNSET:
            field_dict["group"] = group

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.workflow_tag_group_response import WorkflowTagGroupResponse

        d = dict(src_dict)
        name = d.pop("name")

        id = UUID(d.pop("id"))

        organization_id = d.pop("organization_id")

        order = d.pop("order")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_emoji(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        emoji = _parse_emoji(d.pop("emoji", UNSET))

        def _parse_color(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        color = _parse_color(d.pop("color", UNSET))

        def _parse_group_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                group_id_type_0 = UUID(data)

                return group_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        group_id = _parse_group_id(d.pop("group_id", UNSET))

        is_archived = d.pop("is_archived", UNSET)

        def _parse_workflow_count(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        workflow_count = _parse_workflow_count(d.pop("workflow_count", UNSET))

        def _parse_group(data: object) -> None | Unset | WorkflowTagGroupResponse:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                group_type_0 = WorkflowTagGroupResponse.from_dict(data)

                return group_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | WorkflowTagGroupResponse, data)

        group = _parse_group(d.pop("group", UNSET))

        workflow_tag_response = cls(
            name=name,
            id=id,
            organization_id=organization_id,
            order=order,
            created_at=created_at,
            updated_at=updated_at,
            description=description,
            emoji=emoji,
            color=color,
            group_id=group_id,
            is_archived=is_archived,
            workflow_count=workflow_count,
            group=group,
        )

        workflow_tag_response.additional_properties = d
        return workflow_tag_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ReorderRequest")


@_attrs_define
class ReorderRequest:
    """Schema for reordering tags or groups

    Attributes:
        ids (list[UUID]): List of IDs in desired order
    """

    ids: list[UUID]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ids = []
        for ids_item_data in self.ids:
            ids_item = str(ids_item_data)
            ids.append(ids_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ids": ids,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ids = []
        _ids = d.pop("ids")
        for ids_item_data in _ids:
            ids_item = UUID(ids_item_data)

            ids.append(ids_item)

        reorder_request = cls(
            ids=ids,
        )

        reorder_request.additional_properties = d
        return reorder_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="IncludedResource")


@_attrs_define
class IncludedResource:
    """A resource in the JSON:API-style included array.

    Each included resource has a `type` field indicating what kind of resource it is
    (e.g., "workflow", "machine", "pool") and an `id` field with the resource's UUID.
    All other fields from the resource's response model are included.

    Example:
        {
            "type": "workflow",
            "id": "550e8400-e29b-41d4-a716-446655440000",
            "name": "My Workflow",
            "main_prompt": "Do something...",
            "created_at": "2024-01-15T10:30:00Z"
        }

        Attributes:
            type_ (str): Resource type (e.g., 'workflow', 'machine', 'pool')
            id (UUID): Resource UUID
    """

    type_: str
    id: UUID
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        id = str(self.id)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "id": id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        id = UUID(d.pop("id"))

        included_resource = cls(
            type_=type_,
            id=id,
        )

        included_resource.additional_properties = d
        return included_resource

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.workflow_merge_request import WorkflowMergeRequest
from ...models.workflow_response import WorkflowResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    workflow_id: UUID,
    *,
    body: WorkflowMergeRequest,
    idempotency_key: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(idempotency_key, Unset):
        headers["Idempotency-Key"] = idempotency_key

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/workflows/{workflow_id}/merge".format(
            workflow_id=quote(str(workflow_id), safe=""),
        ),
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | WorkflowResponse | None:
    if response.status_code == 200:
        response_200 = WorkflowResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | WorkflowResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    workflow_id: UUID,
    *,
    client: AuthenticatedClient,
    body: WorkflowMergeRequest,
    idempotency_key: str | Unset = UNSET,
) -> Response[HTTPValidationError | WorkflowResponse]:
    """Merge Workflow

     Merge the current workflow into another workflow.

    Updates the target workflow with all data from the source workflow (except version history),
    optionally copies trajectories, and deletes the source workflow.

    Args:
        workflow_id (UUID):
        idempotency_key (str | Unset):  Example: 550e8400-e29b-41d4-a716-446655440000.
        body (WorkflowMergeRequest): Schema for merging workflows

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | WorkflowResponse]
    """

    kwargs = _get_kwargs(
        workflow_id=workflow_id,
        body=body,
        idempotency_key=idempotency_key,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    workflow_id: UUID,
    *,
    client: AuthenticatedClient,
    body: WorkflowMergeRequest,
    idempotency_key: str | Unset = UNSET,
) -> HTTPValidationError | WorkflowResponse | None:
    """Merge Workflow

     Merge the current workflow into another workflow.

    Updates the target workflow with all data from the source workflow (except version history),
    optionally copies trajectories, and deletes the source workflow.

    Args:
        workflow_id (UUID):
        idempotency_key (str | Unset):  Example: 550e8400-e29b-41d4-a716-446655440000.
        body (WorkflowMergeRequest): Schema for merging workflows

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | WorkflowResponse
    """

    return sync_detailed(
        workflow_id=workflow_id,
        client=client,
        body=body,
        idempotency_key=idempotency_key,
    ).parsed


async def asyncio_detailed(
    workflow_id: UUID,
    *,
    client: AuthenticatedClient,
    body: WorkflowMergeRequest,
    idempotency_key: str | Unset = UNSET,
) -> Response[HTTPValidationError | WorkflowResponse]:
    """Merge Workflow

     Merge the current workflow into another workflow.

    Updates the target workflow with all data from the source workflow (except version history),
    optionally copies trajectories, and deletes the source workflow.

    Args:
        workflow_id (UUID):
        idempotency_key (str | Unset):  Example: 550e8400-e29b-41d4-a716-446655440000.
        body (WorkflowMergeRequest): Schema for merging workflows

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | WorkflowResponse]
    """

    kwargs = _get_kwargs(
        workflow_id=workflow_id,
        body=body,
        idempotency_key=idempotency_key,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    workflow_id: UUID,
    *,
    client: AuthenticatedClient,
    body: WorkflowMergeRequest,
    idempotency_key: str | Unset = UNSET,
) -> HTTPValidationError | WorkflowResponse | None:
    """Merge Workflow

     Merge the current workflow into another workflow.

    Updates the target workflow with all data from the source workflow (except version history),
    optionally copies trajectories, and deletes the source workflow.

    Args:
        workflow_id (UUID):
        idempotency_key (str | Unset):  Example: 550e8400-e29b-41d4-a716-446655440000.
        body (WorkflowMergeRequest): Schema for merging workflows

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | WorkflowResponse
    """

    return (
        await asyncio_detailed(
            workflow_id=workflow_id,
            client=client,
            body=body,
            idempotency_key=idempotency_key,
        )
    ).parsed
